#ifndef _h_STRING_
#define _h_STRING_

#include <string.h>
#ifndef _h_TYPES_
# include <cv/types.h>
#endif


/* WARNING:
 *
 * 1. memcpy(,,0) must copy zero bytes
 * 2. String::operator= and String::operator+= uses non-portable way for
 *    assigning character with followed zero-byte
 * 3. string<> use non-portable way for "expanding" String::str
 *
 * NOTE:
 *
 * 1. By /**/ marked not required methods (for optimizations only)
 */


#pragma intrinsic strlen
#pragma intrinsic memcpy
#pragma intrinsic memset


#ifdef __cplusplus


#define defaultStringSize	127

struct			charSeq;
class			String;
typedef const String	&CString;
template<size_t SZ = defaultStringSize> struct string;


#define self		(*this)
#define nullSeq		charSeq(NULL, 0)


/*bool operator == (CString a, charSeq b);*/
/*bool operator == (CString a, CString b);*/
/*bool operator == (CString a, CStr b);*/
/*bool operator == (CString a, char c);*/


/*---------------------------------------------------------*/
struct charSeq{
	size_t len;
	PCStr str;

	charSeq()		:len(0), str(NULL)		{}
	charSeq(const char &c)	:len(1), str(&c)		{}
	charSeq(CStr s)		:len(s ? strlen(s) : 0), str(s)	{}
	charSeq(CString);
	charSeq(CVP p, size_t n):len(n), str(PCStr(p))		{}

/**/	charSeq &operator = (const charSeq &s)
				{ len = s.len; str = s.str; return self; }
	charSeq	&operator ++ ()	{ len--; str++; return self; }
	charSeq &operator += (size_t n)
				{ len -= n; str += n; return self; }
 	RCStr operator [] (size_t i) const { return str[i]; }
	operator CVP () const	{ return str; }
}; /* charSeq */


/*---------------------------------------------------------*/
class String{
    typedef String &Self;
    String(String&) :sz(0) {}	/* NoCopy(String) */
public:
    const size_t sz;
    size_t	 len;

    void store(CVP p, size_t n)	{ memcpy(str, p, setl(n)); str[len] = EOS; }
    Self operator = (const charSeq &s)
				{ store(s, s.len); return self; }
/**/Self operator = (CString s)	{ store(s, s.len); return self; }
/**/Self operator = (char c)	{ if(sz < 2) len = 0; else len = 1,
				  *((word*)str) = word(byte(c)); return self; }

    void add(CVP p, size_t n)	{ register size_t pos = len;
				  memcpy(&str[pos], p, _expand(n) - pos); }
    Self operator += (const charSeq &s)
				{ add(s, s.len); return self; }
/**/Self operator += (CString s){ add(s, s.len); return self; }
/**/Self operator += (char c)	{ if(len + 1 < sz) *((word*)&str[len++])
				  = word(byte(c)); return self; }
    Self	expand(size_t n, char c = ' ')
				{ size_t pos = len; memset(&str[pos], c,
				  _expand(n) - pos); return self; }

    Self	cut(size_t pos, size_t = (size_t)maxUnsigned(size_t));
    Self	replace(size_t pos, charSeq);
    Self	replace(size_t pos, size_t n, charSeq);
    Self	insert(size_t pos, charSeq);
/**/Self	insert(size_t pos, char);
    charSeq	mid(int pos, size_t = (size_t)maxUnsigned(size_t)) const;

    operator CVP	() const	{ return str; }
    operator PCStr	() const	{ return str; }
    operator PStr	()		{ return str; }
    RCStr operator []	(size_t i) const{ return str[i]; }
    RStr operator []	(size_t i)	{ return str[i]; }

    void trunc()		{ len = 0; str[0] = EOS; }
    size_t trunc(size_t n)	{ str[len = (n < len) ? len - n : 0] = EOS;
				  return len; }
    size_t setlen(size_t n)	{ str[setl(n)] = EOS; return len; }
protected:
    char str[2];

    String(size_t sz, const charSeq &s)
				 :sz(sz+1) { str[sz] = EOS; self = s; }
/**/String(size_t sz, CString s) :sz(sz+1) { str[sz] = EOS; self = s; }
/**/String(size_t sz, char c)	 :sz(sz+1) { str[sz] = EOS; self = c; }
/**/String(size_t sz)		 :sz(sz+1), len(0) { str[sz] = str[0] = EOS; }

    size_t setl(size_t n)	{ return len = (n >= sz) ? sz - 1 : n; }
    size_t _expand(size_t n)	{ str[len = (n < sz - len) ? len + n
					: (sz - 1)] = EOS; return len; }
}; /* String */


/*---------------------------------------------------------*/
template<size_t SZ>
struct string :String{
    string(const charSeq &s)	:String(SZ, s)	{}
    string(CString s)		:String(SZ, s)	{}
/**/string(char c)		:String(SZ, c)	{}
/**/string()			:String(SZ)	{}

    string &operator = (const charSeq &s)
				   { String::operator = (s); return self; }
/**/string &operator = (CString s) { String::operator = (s); return self; }
/**/string &operator = (char c)    { String::operator = (c); return self; }
private:
    char place[SZ];
}; /* string<> */


/*=========================================================*/
inline charSeq::charSeq(CString s) :str(s), len(s.len) {}

inline String &String::cut(size_t pos, size_t n){
	if(pos < len)
		if(n < len - pos){
			memmove(&str[pos], &str[pos + n], len - (pos + n));
			trunc(n);
		}else	setlen(pos);
	return self;
}

inline String &String::replace(size_t pos, charSeq s){
	if(pos > len) pos = len;
	memmove(&str[pos], s, s.len > len - pos ?
				_expand(s.len - (len - pos)) - pos : s.len);
	return self;
}

inline String &String::replace(size_t pos, size_t n, charSeq s){
	if(pos > len) pos = len;
	size_t min = s.len < n ? s.len : n;
	if(min > 0) replace(pos, charSeq(s.str, min));
	if(min < sz - pos){
		pos += min;
		if(n < s.len) insert(pos, charSeq(&s[n], s.len - n));
		else if(n > s.len) cut(pos, n - s.len);
	}
	return self;
}

inline String &String::insert(size_t pos, charSeq s){
	if(pos > len) pos = len;
	register size_t tail = _expand(s.len) - pos;
	if(tail > s.len) memmove(&str[pos + s.len], &str[pos], tail - s.len);
	memcpy(&str[pos], s, tail < s.len ? tail : s.len);
	return self;
}

inline String &String::insert(size_t pos, char c){
	if(pos > len) pos = len;
	register size_t tail = _expand(1) - pos;
	if(tail > 1) memmove(&str[pos + 1], &str[pos], tail - 1);
	if(tail > 0) str[pos] = c;
	return self;
}

inline charSeq String::mid(int pos, size_t n) const {
	if(pos < 0) pos += len;
	return (pos < 0 || pos >= len) ? charSeq(NULL, 0)
			: charSeq(&str[pos], (n > len - pos) ? len - pos : n);
}

inline bool operator == (CString a, charSeq b)
	{ return bool(a.len == b.len && (!a.len || !memcmp(a, b, a.len))); }

inline bool operator == (CString a, CString b)
	{ return bool(a.len == b.len && (!a.len || !memcmp(a, b, a.len))); }

inline bool operator == (CString a, CStr b)
	{ return bool((!a.len || !memcmp(a, b, a.len)) && b[a.len] == EOS); }

inline bool operator == (CString a, char c)
	{ return bool(a.len == 1 && a[0] == c); }


#endif /* __cplusplus */


#endif /* _h_STRING_ */
